﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;

namespace Yulin.YWinForm
{
    /// <summary>
    /// 支持多列排序，支持间多列不同的 SortOrder。暂不支持多列自由先后顺序。
    /// </summary>
    public class ListViewItemSorter : IComparer
    {
        public ListView ListView { get; set; }


        public ListViewItemSorter(ListView listView)
        {
            ListView = listView;
        }


        public int Compare(object x, object y)
        {
            ListViewItem itemX = (ListViewItem)x;
            ListViewItem itemY = (ListViewItem)y;

            for (int i = 0; i < ListView.Columns.Count; i++)
            {
                ListViewSortColumn sc = (ListView.Columns[i].Tag as ListViewSortColumn);
                if (!sc.SortEnabled ||
                    sc.SortOrder == SortOrder.None)
                {
                    continue;
                }
                int result = CompareValueWithOrder(itemX.SubItems[i].Text,
                    itemY.SubItems[i].Text,
                    sc.SortDataType,
                    sc.SortOrder);
                if (result != 0)
                {
                    // 在该列上已经见分晓，不用比较后面的列了。
                    return result;
                }
            }

            return 0;
        }


        private int CompareValueWithOrder(string x, string y, ListViewSortDataType listViewSortDataType, SortOrder sortOrder)
        {
            if (sortOrder == SortOrder.Ascending)
            {
                int result = CompareValue(x, y, listViewSortDataType);
                return result;
            }
            else if (sortOrder == SortOrder.Descending)
            {
                int result = CompareValue(x, y, listViewSortDataType);
                if (result > 0)
                {
                    return -1;
                }
                else if (result < 0)
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                return 0;
            }
        }


        private int CompareValue(string x, string y, ListViewSortDataType listViewSortDataType)
        {
            if (listViewSortDataType == ListViewSortDataType.Int32)
            {
                return CompareInt32Value(x, y);
            }
            else
            {
                return CompareStringValue(x, y);
            }
        }


        private int CompareInt32Value(string x, string y)
        {
            int ix = 0;
            int iy = 0;
            int.TryParse(x, out ix);
            int.TryParse(y, out iy);

            return ix.CompareTo(iy);
        }


        private int CompareStringValue(string x, string y)
        {
            return x.CompareTo(y);
        }
    }
}
