﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Yulin.YWinForm
{
    /// <summary>
    /// 排序列，对 ColumnHeader 的扩展属性。
    /// </summary>
    public class ListViewSortColumn
    {
        /// <summary>
        /// 该排序列所属的标头。
        /// </summary>
        public ColumnHeader ColumnHeader { get; set; }

        /// <summary>
        /// 该列是否可排序。
        /// </summary>
        public bool SortEnabled { get; set; }
        /// <summary>
        /// 列的排序数据类型。不同的数据类型有不同排序方法。
        /// </summary>
        public ListViewSortDataType SortDataType { get; set; }
        /// <summary>
        /// 列排序方向。
        /// </summary>
        public SortOrder SortOrder { get; set; }
        
        public const string AscendingMark = " ∧";
        public const string DescendingMark = " ∨";


        public ListViewSortColumn(ColumnHeader columnHeader)
        {
            ColumnHeader = columnHeader;
            SortEnabled = true;
            SortDataType = ListViewSortDataType.String;
            SortOrder = System.Windows.Forms.SortOrder.None;
        }


        /// <summary>
        /// 取消排序。
        /// </summary>
        public void CancelSort()
        {
            SortOrder = System.Windows.Forms.SortOrder.None;

            SetOrderMark();
        }


        /// <summary>
        /// 反转排序。
        /// </summary>
        /// <param name="defaultSortOrder"></param>
        public void ReverseSort(SortOrder defaultSortOrder)
        {
            if (!SortEnabled)
            {
                return;
            }

            if (SortOrder == System.Windows.Forms.SortOrder.Ascending)
            {
                SortOrder = System.Windows.Forms.SortOrder.Descending;
            }
            else if (SortOrder == System.Windows.Forms.SortOrder.Descending)
            {
                SortOrder = System.Windows.Forms.SortOrder.Ascending;
            }
            else
            {
                SortOrder = defaultSortOrder;
            }

            SetOrderMark();
        }


        /// <summary>
        /// 为标头文字设置排序标记。
        /// </summary>
        private void SetOrderMark()
        {
            if (SortOrder == System.Windows.Forms.SortOrder.Ascending)
            {
                ColumnHeader.Text = GetOriginalText() + AscendingMark;
            }
            else if (SortOrder == System.Windows.Forms.SortOrder.Descending)
            {
                ColumnHeader.Text = GetOriginalText() + DescendingMark;
            }
            else
            {
                ColumnHeader.Text = GetOriginalText();
            }
        }


        /// <summary>
        /// 获取不带排序标记的标头文字（如果标头文字本身就带排序标记，则会引起混乱）。
        /// </summary>
        /// <returns></returns>
        private string GetOriginalText()
        {
            string text = ColumnHeader.Text;
            if (String.IsNullOrEmpty(text))
            {
                return "";
            }

            if (text.EndsWith(AscendingMark))
            {
                return text.Substring(0, text.Length - AscendingMark.Length);
            }
            else if (text.EndsWith(DescendingMark))
            {
                return text.Substring(0, text.Length - DescendingMark.Length);
            }
            else
            {
                return text;
            }
        }
    }
}
